/* ============================================================
* ImageFinder plugin for Falkon
* Copyright (C) 2016 Vladislav Tronko <innermous@gmail.com>
*
* This program is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation, either version 3 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program.  If not, see <http://www.gnu.org/licenses/>.
* ============================================================ */
#ifndef IMAGEFINDERSETTINGS_H
#define IMAGEFINDERSETTINGS_H

#include <QDialog>

namespace Ui
{
class ImageFinderSettings;
}

class ImageFinder;

class ImageFinderSettings : public QDialog
{
    Q_OBJECT

public:
    explicit ImageFinderSettings(ImageFinder* finder, QWidget* parent = Q_NULLPTR);

    ~ImageFinderSettings();

private Q_SLOTS:
    void accepted();

private:
    Ui::ImageFinderSettings* ui;
    ImageFinder* m_finder;
};

#endif // IMAGEFINDERSETTINGS_H
